@@ -43,6 +43,7 @@ enum
 static GtkWidget* nm_bt_search_dialog_create_tree_view (NmBtSearchDialog *);
 static void nm_bt_search_dialog_selection_cb (GtkTreeSelection *selection, gpointer user_data);
 static void nm_bt_search_dialog_add_device (NmBtSearchDialog *, gchar *, gchar *);
+static gboolean nm_bt_search_dialog_name_is_in_tree(NmBtSearchDialog *, gchar *);
 
 static void nm_bt_search_dialog_device_found_cb (NmBtUtils *bt_utils, gchar *name,
                                                  gchar *addr, gpointer user_data);
@@ -194,6 +195,42 @@ nm_bt_search_dialog_create_tree_view (NmBtSearchDialog *self)
     return treeview;
 }
 
+static gboolean 
+nm_bt_search_dialog_name_is_in_tree(NmBtSearchDialog *self, gchar *name)
+{
+    GtkTreeModel *model;
+    NmBtSearchDialogPrivate *priv;
+    GtkTreeIter iter;
+    gboolean valid;
+
+    priv = NM_BT_SEARCH_DIALOG_GET_PRIVATE (self);
+
+    model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+
+    if (model != NULL) {
+        valid = gtk_tree_model_get_iter_first (model, &iter);
+        while (valid) {
+            gchar *iter_name;
+            gchar *mac;
+            gtk_tree_model_get(model, &iter,
+                               CONNECTION_DEVICE_COLUMN, &iter_name,
+                               CONNECTION_MAC_COLUMN, &mac,
+                               -1);
+            if (!strcmp(iter_name, name)) {
+                g_free(iter_name);
+                g_free(mac);
+                return TRUE;
+            }
+            
+            g_free(iter_name);
+            g_free(mac);
+            
+            valid = gtk_tree_model_iter_next (model, &iter);
+        }
+    }
+    return FALSE; 
+}
+
 static void
 nm_bt_search_dialog_add_device (NmBtSearchDialog *self, gchar *name, gchar *mac)
 {
@@ -296,7 +333,11 @@ nm_bt_search_dialog_selection_cb (GtkTreeSelection *selection, gpointer user_dat
 static void
 nm_bt_search_dialog_device_found_cb (NmBtUtils *bt_utils, gchar *name, gchar *addr, gpointer user_data)
 {
-    nm_bt_search_dialog_add_device (NM_BT_SEARCH_DIALOG (user_data), name, addr);
+    if ((name != NULL) && (strlen(name) != 0) && 
+        !nm_bt_search_dialog_name_is_in_tree(NM_BT_SEARCH_DIALOG(user_data), name)) {
+            
+        nm_bt_search_dialog_add_device (NM_BT_SEARCH_DIALOG (user_data), name, addr);
+    }  
 }
 
 static void
