@@ -143,6 +143,11 @@ nm_usb_setup_enable (NmUsbSetup *self, gchar *mode)
     else if (!strcmp(mode, NM_USB_MASS_STORAGE_LABEL)) {
         gtk_combo_box_set_active (GTK_COMBO_BOX(priv->checkbutton), NM_USB_MASS_STORAGE);
     }
+#ifdef FREMANTLE
+    else if (!strcmp(mode, NM_USB_CHARGING_ONLY_LABEL)) {
+        gtk_combo_box_set_active (GTK_COMBO_BOX(priv->checkbutton), NM_USB_CHARGING_ONLY);
+    }
+#endif
     else {
 #ifdef DIABLO
     	gtk_combo_box_set_active(GTK_COMBO_BOX(priv->checkbutton), NM_USB_HOST_MODE);
@@ -182,6 +187,15 @@ nm_usb_setup_mode_is_mass_storage (NmUsbSetup *self)
     return (gtk_combo_box_get_active (GTK_COMBO_BOX(priv->checkbutton)) == NM_USB_MASS_STORAGE);
 }
 
+gboolean
+nm_usb_setup_mode_is_charging_only (NmUsbSetup *self)
+{
+    NmUsbSetupPrivate *priv;
+
+    priv = NM_USB_SETUP_GET_PRIVATE (self);
+    return (gtk_combo_box_get_active (GTK_COMBO_BOX(priv->checkbutton)) == NM_USB_CHARGING_ONLY);
+}
+
 static void
 nm_usb_setup_create_widget (NmUsbSetup *self)
 {
@@ -201,7 +215,8 @@ nm_usb_setup_create_widget (NmUsbSetup *self)
     gtk_combo_box_insert_text (GTK_COMBO_BOX(priv->checkbutton), NM_USB_HOST_MODE, NM_USB_HOST_MODE_LABEL);
 #endif
 #ifdef FREMANTLE
-	gtk_combo_box_insert_text(priv->checkbutton, NM_USB_PC_SUITE, NM_USB_PC_SUITE_LABEL);
+	gtk_combo_box_insert_text(GTK_COMBO_BOX(priv->checkbutton), NM_USB_PC_SUITE, NM_USB_PC_SUITE_LABEL);
+	gtk_combo_box_insert_text(GTK_COMBO_BOX(priv->checkbutton), NM_USB_CHARGING_ONLY, NM_USB_CHARGING_ONLY_LABEL);
 #endif	
     /* DHCPD server */
     priv->dhcpd_label = gtk_label_new(_("DHCP Server: "));
@@ -398,7 +413,12 @@ nm_usb_setup_apply_changes (NmConnWidget *self)
         
     } else if (nm_usb_setup_mode_is_mass_storage (usbsetup)) {
         nm_usb_utils_put_in_otg_mode();
+    } 
+#ifdef FREMANTLE    
+    else if (nm_usb_setup_mode_is_charging_only (usbsetup)) {
+        nm_usb_utils_put_in_charging_only_mode();
     }
+#endif
 
     return TRUE;
 }
@@ -502,9 +522,15 @@ usb_setup_check_enabled (NmUsbSetup *self)
     } else if (nm_usb_utils_is_host_mode ()) {
     	
     	nm_usb_setup_enable (self, NM_USB_HOST_MODE_LABEL);
+        return FALSE;
+    } else if (nm_usb_utils_is_charging_only ()) {
+    	
+    	nm_usb_setup_enable (self, NM_USB_CHARGING_ONLY_LABEL);
+        return FALSE;
     } else {
     	
         nm_usb_setup_enable (self, NM_USB_MASS_STORAGE_LABEL);
+        return FALSE;
     }
     return TRUE;
 }
